module bispectra
  !PDM 2016: Module defining stadard bispectra; also
  !including proposed templates
  use settings
  implicit none
  
contains


  real function Blocal(k)

    real(dl) :: k(3)
    real(dl) :: x,y,z
    !shape is smmetruc in k1, k2, k3, which will reduce the number of terms
    !necessary in the overlap computation
    use_symmetry = .true.
    x = k(1)/kmax
    y = k(2)/kmax
    z = k(3)/kmax
    !Blocal  = 2.d0/(x*y)**3 + 2.d0/(x*z)**3 + &
    !     2.d0/(y*z)**3
    Blocal  = 2.d0*z**2/(x*y) + 2.d0*y**2/(x*z) + &
         2.d0*x**2/(y*z)
  end function Blocal

  real function Bequil(k)

    real(dl) :: k(3)
    real(dl) :: x,y,z
    use_symmetry = .true.
    x = k(1)/kmax
    y = k(2)/kmax
    z = k(3)/kmax
    !Bequil  =  6.d0*(-1.d0/(x*y)**3 - 1.d0/(x*z)**3 - &
    !     1.d0/(y*z)**3 - 2.d0/(x*y*z)**2 + &
    !     1.d0/(x*y**2*z**3) + 1.d0/(y*x**2*z**3) + &
    !     1.d0/(x*z**2*y**3) + 1.d0/(z*y**2*x**3) + &
    !     1.d0/(z*x**2*y**3) + 1.d0/(y*z**2*x**3))
    Bequil  =  6.d0*(-1.d0*z**2/(x*y) - 1.d0*y**2/(x*z) - &
         1.d0*x**2/(y*z) - 2.d0 + &
         1.d0*x/z + 1.d0*y/x + &
         1.d0*x/y+ 1.d0*z/x + &
         1.d0*z/y + 1.d0*y/z)
  end function Bequil

  real function Bortho(k)

    real(dl) :: k(3)
    real(dl) :: x,y,z
    x = k(1)/kmax
    y = k(2)/kmax
    z = k(3)/kmax
    use_symmetry = .true.
    !Bortho  =  6.d0*(-3.d0/(x*y)**3 - 3.d0/(x*z)**3 - &
    !     3.d0/(y*z)**3 - 8.d0/(x*y*z)**2 + &
    !     3.d0/(x*y**2*z**3) + 3.d0/(y*x**2*z**3) + &
    !     3.d0/(x*z**2*y**3) + 3.d0/(z*y**2*x**3) + &
    !     3.d0/(z*x**2*y**3) + 3.d0/(y*z**2*x**3))
    Bortho  =  6.d0*(-3.d0*z**2/(x*y) - 3.d0*y**2/(x*z) - &
         3.d0*x**2/(y*z) - 8.d0 + &
         3.d0*x/z + 3.d0*y/x + &
         3.d0*x/y+ 3.d0*z/x + &
         3.d0*z/y + 3.d0*y/z)
  end function Bortho

  real function Btemp1(k,mu,b)
    real(dl) :: k(3)
    real(dl) :: mu,b(2)
    real(dl) :: x,y,z
    real(dl) :: alpha(3)
    x = k(1)/kmax
    y = k(2)/kmax
    z = k(3)/kmax
    use_symmetry = .true.
    alpha(1) = (k(1)+k(2))/k(3)
    alpha(2) = (k(1)+k(3))/k(2)
    alpha(3) = (k(2)+k(3))/k(1)
    
    Btemp1 = ((1.d0-exp(-((alpha(1)-1.d0)/b(1))**(b(2))))*alpha(1)**(-1/2)*sin(mu*log(alpha(1)/2.d0))+ &
         (1.d0-exp(-((alpha(2)-1.d0)/b(1))**(b(2))))*alpha(2)**(-1/2)*sin(mu*log(alpha(2)/2.d0)) + &
         (1.d0-exp(-((alpha(3)-1.d0)/b(1))**(b(2))))*alpha(3)**(-1/2)*sin(mu*log(alpha(3)/2.d0)))

  end function Btemp1
  
  real function Btemp2(k,mu,c)
    real(dl) :: k(3)
    real(dl) :: mu,c
    real(dl) :: x,y,z
    real(dl) :: alpha(3)

    real(dl) :: Bpart(3)
    x = k(1)/kmax
    y = k(2)/kmax
    z = k(3)/kmax
    use_symmetry = .true.
    alpha(1) = (k(1)+k(2))/k(3)
    alpha(2) = (k(1)+k(3))/k(2)
    alpha(3) = (k(2)+k(3))/k(1)

    if (alpha(1) .ge. c) then 
       Bpart(1) = alpha(1)**(-1/2)*sin(mu*log(alpha(1)/2.d0))
    else
       Bpart(1) = 0
    endif
    if (alpha(2) .ge. c) then 
       Bpart(2) = alpha(2)**(-1/2)*sin(mu*log(alpha(2)/2.d0))
    else
       Bpart(2) = 0
    endif
    if (alpha(3) .ge. c) then 
       Bpart(3) = alpha(3)**(-1/2)*sin(mu*log(alpha(3)/2.d0))
    else
       Bpart(3) = 0
    endif

    Btemp2 = sum(Bpart)

  end function Btemp2
end module bispectra
