module overlap

  use settings
  use bispectra
  implicit none 
contains 

  subroutine innerBB(shape1,shape2, y)

    integer :: shape1, shape2
    integer :: i,j,l
    real(dl) :: k(3)
    real(dl) :: B1temp, B2temp
    real(dl) :: dk(3), res(4)
    real(dl) :: sum1
    real(dl) :: y
    real(dl) :: weightf

    sum1 = 0.d0
    !$OMP PARAllEl DO DEFAULT(SHARED), &
    !$OMP PRIVATE(k,i,j,l,B1temp,B2temp) &
    !$OMP REDUCTION(+:sum1)
    do i = 1, ikmid
       k(1) = samplek(i) !x

       do j  = i, ikmax - i
          k(2) = samplek(j) !y

          do l = j-i+1, i+j - 2
             k(3) = samplek(l)

             if (shape1 .eq. 1) then
               B1temp = Blocal(k)
                !write(*,*) B1temp
             elseif (shape1 .eq. 2) then
                B1temp = Bequil(k)
             elseif(shape1 .eq. 3) then
                B1temp  = Bortho(k)
             elseif(shape1 .eq. 4) then
                B1temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape1 .eq. 5) then
                B1temp  = Btemp2(k,temp_mu,temp_c)
             else
                stop 'shape not known'
             endif
             if (shape2 .eq. 1) then
                B2temp = Blocal(k)
             elseif (shape2 .eq. 2) then
                B2temp = Bequil(k)
             elseif(shape2 .eq. 3) then
                B2temp  = Bortho(k)
             elseif(shape2 .eq. 4) then
                B2temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape2 .eq. 5) then
                B2temp  = Btemp2(k,temp_mu,temp_c)
             else
                !stop 'shape not known'
             endif
             if(want_weight) then
                weightf = 1.d0/(k(1)+k(2)+k(3))
             else
                weightf = 1.d0
             endif
             
             dk(3) = sampledk(l)
             dk(2) = sampledk(j)
             dk(1) = sampledk(i)
             sum1 = sum1+ weightf*B1temp*B2temp*dk(3)*dk(2)*dk(1)

          enddo !k3

       enddo !k2

    enddo !k1
    !$OMP END PARALLEL DO
    write(*,*) sum1
    res(1) = sum1

    if (use_symmetry) then
       res(2) = sum1
    else
    sum1 = 0.d0
    
    !$OMP PARAllEl DO DEFAULT(SHARED), &
    !$OMP PRIVATE(k,i,j,l,B1temp,B2temp) &
    !$OMP REDUCTION(+:sum1)    
     do i = 1, ikmid
       k(2) = samplek(i)
       do j  = i, ikmax - i+1
          k(1) = samplek(j)
          do l = j-i+1, j+i - 1
             k(3) = samplek(l)

             if (shape1 .eq. 1) then
                B1temp = Blocal(k)
                
             elseif (shape1 .eq. 2) then
                B1temp = Bequil(k)
             elseif(shape1 .eq. 3) then
                B1temp  = Bortho(k)
             elseif(shape1 .eq. 4) then
                B1temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape1 .eq. 5) then
                B1temp  = Btemp2(k,temp_mu,temp_c)
             else
                stop 'shape not known'
             endif
             if (shape2 .eq. 1) then
                B2temp = Blocal(k)
             elseif (shape2 .eq. 2) then
                B2temp = Bequil(k)
             elseif(shape2 .eq. 3) then
                B2temp  = Bortho(k)
             elseif(shape2 .eq. 4) then
                B2temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape2 .eq. 5) then
                B2temp  = Btemp2(k,temp_mu,temp_c)
             else
                stop 'shape not known'
             endif
             if(want_weight) then
                weightf = 1.d0/(k(1)+k(2)+k(3))
             else
                weightf = 1.d0
             endif
             dk(3) = sampledk(l)
             dk(2) = sampledk(j)
             dk(1) = sampledk(i)
             sum1 = sum1+ weightf*B1temp*B2temp*dk(3)*dk(2)*dk(1)

          enddo !k3

       enddo !k2

    enddo !k1
    !$OMP END PARALLEL DO
    write(*,*) sum1
    res(2) = sum1
    endif
    sum1 = 0.d0
    !$OMP PARAllEl DO DEFAULT(SHARED), &
    !$OMP PRIVATE(k,i,j,l,B1temp,B2temp) &
    !$OMP REDUCTION(+:sum1)   
    do i = ikmid, ikmax-1
       k(1) = samplek(i)
       do j  = ikmax - i + 1, i
          k(2) = samplek(j)
          do l = i-j+1, ikmax-1
             k(3) = samplek(l)

             if (shape1 .eq. 1) then
                B1temp = Blocal(k)
             elseif (shape1 .eq. 2) then
                B1temp = Bequil(k)
             elseif(shape1 .eq. 3) then
                B1temp  = Bortho(k)
             elseif(shape1 .eq. 4) then
                B1temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape1 .eq. 5) then
                B1temp  = Btemp2(k,temp_mu,temp_c)
             else
                stop 'shape not known'
             endif
             if (shape2 .eq. 1) then
                B2temp = Blocal(k)
             elseif (shape2 .eq. 2) then
                B2temp = Bequil(k)
             elseif(shape2 .eq. 3) then
                B2temp  = Bortho(k)
             elseif(shape2 .eq. 4) then
                B2temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape2 .eq. 5) then
                B2temp  = Btemp2(k,temp_mu,temp_c)
             else
                stop 'shape not known'
             endif
             if(want_weight) then
                weightf = 1.d0/(k(1)+k(2)+k(3))
             else
                weightf = 1.d0
             endif
             dk(3) = sampledk(l)
             dk(2) = sampledk(j)
             dk(1) = sampledk(i)
             sum1 = sum1+ weightf*B1temp*B2temp*dk(3)*dk(2)*dk(1)

          enddo !k3

       enddo !k2

    enddo !k1
    !$OMP END PARALLEL DO
    write(*,*) "bla",sum1
    res(3) = sum1
    
    if (use_symmetry) then 
       res(4)  = sum1
    else
       
    sum1 = 0.d0
    !$OMP PARAllEl DO DEFAULT(SHARED), &
    !$OMP PRIVATE(k,i,j,l,B1temp,B2temp) &
    !$OMP REDUCTION(+:sum1)  
    do i = ikmid, ikmax-1
       k(2) = samplek(i)
       do j  = ikmax - i + 1, i 
          k(1) = samplek(j)
          do l = i-j+1, ikmax-1
             k(3) = samplek(l)

             if (shape1 .eq. 1) then
                B1temp = Blocal(k)
             elseif (shape1 .eq. 2) then
                B1temp = Bequil(k)
             elseif(shape1 .eq. 3) then
                B1temp  = Bortho(k)
             elseif(shape1 .eq. 4) then
                B1temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape1 .eq. 5) then
                B1temp  = Btemp2(k,temp_mu,temp_c)
             else
                stop 'shape not known'
             endif
             if (shape2 .eq. 1) then
                B2temp = Blocal(k)
             elseif (shape2 .eq. 2) then
                B2temp = Bequil(k)
             elseif(shape2 .eq. 3) then
                B2temp  = Bortho(k)
             elseif(shape2 .eq. 4) then
                B2temp  = Btemp1(k,temp_mu,temp_b)
             elseif(shape2 .eq. 5) then
                B2temp  = Btemp2(k,temp_mu,temp_c)
             else
                stop 'shape not known'
             endif
             if(want_weight) then
                weightf = 1.d0/(k(1)+k(2)+k(3))
             else
                weightf = 1.d0
             endif
             dk(3) = sampledk(l)
             dk(2) = sampledk(j)
             dk(1) = sampledk(i)
             sum1 = sum1+ weightf*B1temp*B2temp*dk(3)*dk(2)*dk(1)

          enddo !k3

       enddo !k2

    enddo !k1
    !$OMP END PARALLEL DO
    write(*,*) sum1
    res(4) = sum1
    endif
    y = sum(res)

  end subroutine innerBB

  subroutine cosBB(shape1, shape2, mue, x)
    integer :: shape1, shape2
    real(dl) :: x
    real(dl) :: mue(2)
    real(dl) :: norm_y1, norm_y2, y1y2
    if (use_2D) then
       call innerBB_2D(shape1, shape1, mue(1), mue(1), norm_y1)
       call innerBB_2D(shape2, shape2, mue(2), mue(2), norm_y2)
       call innerBB_2D(shape1, shape2, mue(1), mue(2), y1y2)
    else       
       call innerBB(shape1, shape1, norm_y1)
       call innerBB(shape2, shape2, norm_y2)
       call innerBB(shape1, shape2, y1y2)
    endif
    

    x = y1y2/sqrt(norm_y1)/sqrt(norm_y2)
  end subroutine cosBB

  subroutine innerBB_2D(shape1,shape2, mu1, mu2, y)

    integer :: shape1, shape2
    integer :: i,j,l
    real(dl) :: mu1,mu2
    real(dl) :: k(3)
    real(dl) :: B1temp, B2temp
    real(dl) :: integrand(3), dk(3), res(4)
    real(dl) :: sum1
    real(dl) :: y
    real(dl) :: weightf

    sum1 = 0.d0
    k(1) = 1.d0

    !$OMP PARAllEl DO DEFAULT(SHARED), &
    !$OMP PRIVATE(k,j,l,B1temp,B2temp) &
    !$OMP REDUCTION(+:sum1)

    do j  = ikmid, ikmax-1
       k(2) = samplek(j)
       k(1) = 1.d0
       do l = ikmax-j+1, j 
          k(3) = samplek(l)

          if (shape1 .eq. 1) then
             B1temp = Blocal(k)
             !write(*,*) B1temp
          elseif (shape1 .eq. 2) then
             B1temp = Bequil(k)
          elseif(shape1 .eq. 3) then
             B1temp  = Bortho(k)
          elseif(shape1 .eq. 4) then
             B1temp  = Btemp1(k,mu1,temp_b)
          elseif(shape1 .eq. 5) then
             B1temp  = Btemp2(k,mu1,temp_c)
          else
             stop 'shape not known'
          endif
          if (shape2 .eq. 1) then
             B2temp = Blocal(k)
          elseif (shape2 .eq. 2) then
             B2temp = Bequil(k)
          elseif(shape2 .eq. 3) then
             B2temp  = Bortho(k)
          elseif(shape2 .eq. 4) then
             B2temp  = Btemp1(k,mu2,temp_b)
          elseif(shape2 .eq. 5) then
             B2temp  = Btemp2(k,mu2,temp_c)
          else
             !stop 'shape not known'
          endif
          if(want_weight) then
             weightf = 1.d0/(k(1)+k(2)+k(3))
          else
             weightf = 1.d0
          endif

          dk(3) = sampledk(l)
          dk(2) = sampledk(j)
          sum1 = sum1+ weightf*B1temp*B2temp*dk(3)*dk(2)
          !write(*,*) j,l,B1temp,B2temp
       enddo !k3

    enddo !k2

    !$OMP END PARALLEL DO
    write(*,*) sum1
    y = sum1

  end subroutine innerBB_2D
  
  
end module overlap
